package it.polito.bigdata.spark.sparkmllib;

import org.apache.spark.api.java.*;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.ml.Pipeline;
import org.apache.spark.ml.PipelineModel;
import org.apache.spark.ml.PipelineStage;
import org.apache.spark.ml.linalg.Vector;
import org.apache.spark.ml.linalg.Vectors;
import org.apache.spark.ml.clustering.KMeans;

public class SparkDriver {

	@SuppressWarnings("resource")
	public static void main(String[] args) {

		String inputFile;
		String outputPath;

		inputFile = args[0];
		outputPath = args[1];

		// Create a Spark Session object and set the name of the application
		// We use some Spark SQL transformation in this program
		SparkSession ss = SparkSession.builder().appName("MLlib - K-means").getOrCreate();

		// Create a Java Spark Context from the Spark Session
		// When a Spark Session has already been defined this method
		// is used to create the Java Spark Context
		JavaSparkContext sc = new JavaSparkContext(ss.sparkContext());

		// Read training data from a textual file
		// Each line contains 3 double values /the input records are
		// characterized by three attributes)
		// E.g., 1.0,5.0,4.5
		JavaRDD<String> inputData = sc.textFile(inputFile);

		// Map each input record/data point of the input file to a InputRecord object
		// InputRecord is a class defined in this application. It is used to 
		// represent records with one single attribute called features (data type: Vector)
		JavaRDD<InputRecord> inputRDD = inputData.map(record -> {
			String[] fields = record.split(",");

			// The three cells of fields contain the (numerical) values of the
			// three input attributes
			// Create an array of doubles containing those three values
			double[] attributesValues = new double[3];

			attributesValues[0] = Double.parseDouble(fields[0]);
			attributesValues[1] = Double.parseDouble(fields[1]);
			attributesValues[2] = Double.parseDouble(fields[2]);

			// Create a dense vector based on the content of attributesValues
			Vector attrValues = Vectors.dense(attributesValues);

			// Return the Vector associated with the current line
			return new InputRecord(attrValues);
		});

		// Create a DataFrame based on the input data.
		Dataset<Row> data = ss.createDataFrame(inputRDD, InputRecord.class).cache();

		// Create a k-means object.
		// k-means is an Estimator that is used to
		// create a k-means algorithm
		KMeans km = new KMeans();

		// Set the value of k
		km.setK(2);

		// Define the pipeline that is used to cluster
		// the input data
		// In this case the pipeline contains one single stage/step (the model
		// generation step).
		Pipeline pipeline = new Pipeline().setStages(new PipelineStage[] { km });

		// Execute the pipeline on the data to build the
		// clustering model
		PipelineModel model = pipeline.fit(data);

		// Now the clustering model can be applied on the data
		// to assign them to a cluster (i.e., assign a cluster id)
		// The returned DataFrame has the following schema (attributes)
		// - features: vector (values of the attributes)
		// - prediction: double (the predicted cluster id)
		Dataset<Row> clusteredData = model.transform(data);

		// Save the result in an HDFS file
		JavaRDD<Row> clusteredDataRDD = clusteredData.javaRDD();
		clusteredDataRDD.saveAsTextFile(outputPath);

		// Close the Spark Context object
		sc.close();
	}

}
